using System.Text;
using Microsoft.CodeAnalysis;
using Microsoft.DevSkim.LanguageProtoInterop;

namespace Microsoft.DevSkim.VisualStudio
{
    [Generator]
    public class UpdateSettingsGenerator : ISourceGenerator
    {
        public void Initialize(GeneratorInitializationContext context)
        {
        }

        public void Execute(GeneratorExecutionContext context)
        {
            StringBuilder source = new StringBuilder();
            source.Append($@"// <auto-generated/>
    namespace Microsoft.DevSkim.VisualStudio
    {{
        using System;
        using Microsoft.DevSkim.LanguageProtoInterop;
        using System.Collections.Generic;

        internal partial class VisualStudioSettingsManager
        {{
            partial void UpdateSettings(string propertyName)
            {{
                switch(propertyName)
                {{
");

            string baseIndentation = "                ";
            var props = typeof(IDevSkimOptions).GetProperties().Select(x => (x.Name, x.PropertyType));
            foreach (var prop in props)
            {
                var typeNameString = GetTypeNameString(prop.PropertyType);
                source.Append($@"{baseIndentation}case ""{prop.Name}"":
                        {{
                            var res = Get<{typeNameString}>(propertyName);
                            if (res.Item1 == ValueResultEnum.Success)
                            {{
                                _currentSettings.{prop.Name} = res.Item2{ConditionallyUseNullCoalescing(typeNameString)}
                            }}
                            break;
                        }}                    
");
            }

            source.Append($@"{baseIndentation}default: break;
                }}
            }}
        }}
    }}");
            context.AddSource("VisualStudioSettingsManager.g.cs", source.ToString());
        }

        private string ConditionallyUseNullCoalescing(string typeNameString)
        {
            return typeNameString.StartsWith("List") ? $" ?? new {typeNameString}();" : ";";
        }

        private string GetTypeNameString(Type propPropertyType)
        {
            return propPropertyType.Name switch
            {
                "String" => "string",
                "List`1" => $"List<{GetTypeNameString(propPropertyType.GetGenericArguments()[0])}>",
                "Boolean" => "bool",
                "Int32" => "int",
                "CommentStylesEnum" => "CommentStylesEnum",
                _ => "string"
            };
        }
    }
}